#include "../inc/fire.h"
#include "../inc/color.h"
#include <stdlib.h>
#include <stdint.h>

color_ti_t leds[COLOR_LED_COUNT];
uint16_t wood = 0;
uint16_t oil = 0;
uint8_t intensities[COLOR_LED_COUNT];

static uint8_t rand_256(void);
static int16_t average(int16_t arg1, int16_t arg2);
static int16_t min(int16_t arg1, int16_t arg2);
static int16_t max(int16_t arg1, int16_t arg2);
static int16_t integrate(int16_t value, int16_t target, uint8_t div);
static int16_t abs(int16_t value);
static int16_t sign(int16_t value);

void fire_ignite(void) {
	if (wood) {
		intensities[rand_256() / (256 / COLOR_LED_COUNT)] = 0xff;
	}
}

void fire_add_wood(void) {
	wood = min(1000, wood + rand_256() / 4 + 200);
}

void fire_add_oil(void) {
	oil = rand_256() * 2 + 100;
}

uint16_t fire_tick(void) {
	static uint8_t prescaler = 0;
	static uint16_t calc_prescaler = FIRE_CALC_PRESCALER;
	uint16_t intensity = 0;
	static uint8_t intensities_smooth[COLOR_LED_COUNT] = {0};

	if (!prescaler) {
		for (uint8_t i = 0; i < COLOR_LED_COUNT; ++i) {
			leds[i].intensity = min(255, intensities_smooth[i] * 4 + oil * 4);
			leds[i].temperature = min(rand_256(), oil ? 255 : max(wood ? 100 : 10, leds[i].intensity));
			intensity += leds[i].intensity;
		}
		prescaler = oil ? 1 : 4;
		if (oil) --oil;
	} else {
		--prescaler;
		intensity = 255;
	}
	intensity = intensity / COLOR_LED_COUNT;

	if (!calc_prescaler) {
		calc_prescaler = FIRE_CALC_PRESCALER;
		for (uint8_t i = 0; i < COLOR_LED_COUNT; ++i) {
			uint8_t ileft, iright;
			if (i) {
				ileft = intensities[i - 1];
			} else {
				ileft = intensities[i];
			}
			if (i < COLOR_LED_COUNT - 1) {
				iright = intensities[i + 1];
			} else {
				iright = intensities[i];
			}
			intensities[i] = integrate(intensities[i], average(ileft, iright), 16);
			if (wood) {
				--wood;
				if (intensities[i]) intensities[i] = min(255, intensities[i] + 2);
			}
			if (intensities[i]) --intensities[i];
		}
	} else {
		--calc_prescaler;
	}

	if ((calc_prescaler & 0x000f) == 0) {
		for (uint8_t i = 0; i < COLOR_LED_COUNT; ++i) {
			if (intensities_smooth[i] > intensities[i]) {
				--intensities_smooth[i];
			} else if (intensities_smooth[i] < intensities[i]) {
				++intensities_smooth[i];
			}
		}
	}

	color_tick(leds);
	return intensity || wood;
}

static uint8_t rand_256(void) {
	return rand() / (uint8_t)(((uint32_t)RAND_MAX + 1) / 256);
}

static int16_t average(int16_t arg1, int16_t arg2) {
	return (arg1 + arg2) / 2;
}

static int16_t min(int16_t arg1, int16_t arg2) {
	if (arg1 < arg2) {
		return arg1;
	} else {
		return arg2;
	}
}

static int16_t max(int16_t arg1, int16_t arg2) {
	if (arg2 < arg1) {
		return arg1;
	} else {
		return arg2;
	}
}

static int16_t abs(int16_t value) {
	if (value < 0) {
		return -value;
	} else {
		return value;
	}
}

static int16_t sign(int16_t value) {
	if (value < 0) {
		return -1;
	} else if (!value) {
		return 0;
	} else {
		return 1;
	}
}

static int16_t integrate(int16_t value, int16_t target, uint8_t div) {
	int16_t d = target - value;
	if (abs(d) < div) {
		d = sign(d);
	} else {
		d /= div;
	}
	return value + d;
}




