#include "../inc/pwm.h"
#include "../inc/init.h"
#include <avr/io.h>
#include <avr/interrupt.h>
#include <stdint.h>
#include <string.h>

volatile uint8_t pwm_values[PWM_CHANNELS];
volatile uint8_t sync;

const uint16_t channel_mask[PWM_CHANNELS] = {
	_BV(PIN_EN1), _BV(PIN_EN2), _BV(PIN_EN3),
	_BV(PIN_EN4), _BV(PIN_EN5), _BV(PIN_EN6),
	_BV(PIN_EN7) << 8, _BV(PIN_EN8) << 8, _BV(PIN_EN9) << 8,
	_BV(PIN_EN10) << 8, _BV(PIN_EN11) << 8, _BV(PIN_EN12) << 8,
};

void pwm_init(void) {
	TCCR2 = _BV(CS22) | _BV(CS21);
	TIMSK |= _BV(OCIE2);
}

void pwm_update(uint8_t values[PWM_CHANNELS]) {
	while (sync);		//Wait for the rising edge of sync
	while (!sync);
	cli();
	if (values) {
		for (uint8_t i = 0; i < PWM_CHANNELS; ++i) {
			pwm_values[i] = values[i] & 0xfe;
		}
	} else {
		for (uint8_t i = 0; i < PWM_CHANNELS; ++i) {
			pwm_values[i] = 0;
		}
	}
	sei();
}


ISR(TIMER2_COMP_vect) {
	uint8_t ocr = OCR2, diff;
	uint16_t setmask = 0, clearmask = 0;

	if (!ocr) {
		clearmask = PWM_MASK_ALL;
		diff = 255;
	} else {
		diff = 0 - ocr;		//256 - ocr
	}

	for (uint8_t i = 0; i < PWM_CHANNELS; ++i) {
		uint8_t d;
		d = pwm_values[i] - ocr;
		if (!d) {
			setmask |= channel_mask[i];
		} else {
			if (d < diff) diff = d;
		}
	}
	ocr += diff;
	if (!ocr) {
		sync = 1;
	} else {
		sync = 0;
	}
	OCR2 = ocr;

	clearmask = ~clearmask;
	clearmask |= setmask;
	uint8_t s1 = setmask & 0x00ff, c1 = clearmask & 0x00ff;
	uint8_t s2 = setmask >> 8, c2 = clearmask >> 8;
	RPO_EN_1 = (RPO_EN_1 & c1) | s1;
	RPO_EN_2 = (RPO_EN_2 & c2) | s2;
}





